@@ -4,7 +4,7 @@
 #include "database.h"
 
 #include "connection.h"
-#include "logging.h"
+#include "logging_categories_p.h"
 
 #include "e2ee/qolmaccount.h"
 #include "e2ee/qolminboundsession.h"
@@ -20,9 +20,8 @@
 using namespace Quotient;
 
 Database::Database(const QString& userId, const QString& deviceId,
-                   PicklingKey&& picklingKey, QObject* parent)
-    : QObject(parent)
-    , m_userId(userId)
+                   PicklingKey&& picklingKey)
+    : m_userId(userId)
     , m_deviceId(deviceId)
     , m_picklingKey(std::move(picklingKey))
 {
@@ -56,30 +55,29 @@ int Database::version()
         if (ok)
             return value;
     } else {
-        qCritical() << "Failed to check database version";
+        qCritical(DATABASE) << "Failed to check database version";
     }
     return -1;
 }
 
-QSqlQuery Database::execute(const QString &queryString)
+QSqlQuery Database::execute(const QString& queryString)
 {
     auto query = database().exec(queryString);
     if (query.lastError().type() != QSqlError::NoError) {
-        qCritical() << "Failed to execute query";
-        qCritical() << query.lastQuery();
-        qCritical() << query.lastError();
+        qCritical(DATABASE) << "Failed to execute query";
+        qCritical(DATABASE) << query.lastQuery();
+        qCritical(DATABASE) << query.lastError();
     }
     return query;
 }
 
-QSqlQuery Database::execute(QSqlQuery &query)
+void Database::execute(QSqlQuery& query)
 {
     if (!query.exec()) {
-        qCritical() << "Failed to execute query";
-        qCritical() << query.lastQuery();
-        qCritical() << query.lastError();
+        qCritical(DATABASE) << "Failed to execute query";
+        qCritical(DATABASE) << query.lastQuery();
+        qCritical(DATABASE) << query.lastError();
     }
-    return query;
 }
 
 void Database::transaction()
@@ -201,7 +199,7 @@ void Database::clear()
 
 }
 
-void Database::saveOlmSession(const QString& senderKey,
+void Database::saveOlmSession(const QByteArray& senderKey,
                               const QOlmSession& session,
                               const QDateTime& timestamp)
 {
@@ -215,19 +213,19 @@ void Database::saveOlmSession(const QString& senderKey,
     commit();
 }
 
-UnorderedMap<QString, std::vector<QOlmSession>> Database::loadOlmSessions()
+UnorderedMap<QByteArray, std::vector<QOlmSession>> Database::loadOlmSessions()
 {
     auto query = prepareQuery(QStringLiteral(
         "SELECT * FROM olm_sessions ORDER BY lastReceived DESC;"));
     transaction();
     execute(query);
     commit();
-    UnorderedMap<QString, std::vector<QOlmSession>> sessions;
+    UnorderedMap<QByteArray, std::vector<QOlmSession>> sessions;
     while (query.next()) {
         if (auto&& expectedSession =
                 QOlmSession::unpickle(query.value("pickle"_ls).toByteArray(),
                                       m_picklingKey)) {
-            sessions[query.value("senderKey"_ls).toString()].emplace_back(
+            sessions[query.value("senderKey"_ls).toByteArray()].emplace_back(
                 std::move(*expectedSession));
         } else
             qCWarning(E2EE)
@@ -236,7 +234,7 @@ UnorderedMap<QString, std::vector<QOlmSession>> Database::loadOlmSessions()
     return sessions;
 }
 
-UnorderedMap<QString, QOlmInboundGroupSession> Database::loadMegolmSessions(
+UnorderedMap<QByteArray, QOlmInboundGroupSession> Database::loadMegolmSessions(
     const QString& roomId)
 {
     auto query = prepareQuery(QStringLiteral("SELECT * FROM inbound_megolm_sessions WHERE roomId=:roomId;"));
@@ -244,11 +242,11 @@ UnorderedMap<QString, QOlmInboundGroupSession> Database::loadMegolmSessions(
     transaction();
     execute(query);
     commit();
-    UnorderedMap<QString, QOlmInboundGroupSession> sessions;
+    UnorderedMap<QByteArray, QOlmInboundGroupSession> sessions;
     while (query.next()) {
         if (auto&& expectedSession = QOlmInboundGroupSession::unpickle(
                 query.value("pickle"_ls).toByteArray(), m_picklingKey)) {
-            const auto sessionId = query.value("sessionId"_ls).toString();
+            const auto sessionId = query.value("sessionId"_ls).toByteArray();
             if (const auto it = sessions.find(sessionId); it != sessions.end()) {
                 qCritical(DATABASE) << "More than one inbound group session "
                                        "with the same session id"
@@ -261,9 +259,9 @@ UnorderedMap<QString, QOlmInboundGroupSession> Database::loadMegolmSessions(
                 sessions.erase(it);
             }
             expectedSession->setOlmSessionId(
-                query.value("olmSessionId"_ls).toString());
+                query.value("olmSessionId"_ls).toByteArray());
             expectedSession->setSenderId(query.value("senderId"_ls).toString());
-            sessions.try_emplace(query.value("sessionId"_ls).toString(),
+            sessions.try_emplace(query.value("sessionId"_ls).toByteArray(),
                                  std::move(*expectedSession));
         } else
             qCWarning(E2EE) << "Failed to unpickle megolm session:"
@@ -315,12 +313,12 @@ std::pair<QString, qint64> Database::groupSessionIndexRecord(const QString& room
     return {query.value("eventId"_ls).toString(), query.value("ts"_ls).toLongLong()};
 }
 
-QSqlDatabase Database::database()
+QSqlDatabase Database::database() const
 {
     return QSqlDatabase::database("Quotient_"_ls + m_userId);
 }
 
-QSqlQuery Database::prepareQuery(const QString& queryString)
+QSqlQuery Database::prepareQuery(const QString& queryString) const
 {
     QSqlQuery query(database());
     query.prepare(queryString);
@@ -432,7 +430,7 @@ QMultiHash<QString, QString> Database::devicesWithoutKey(
     return devices;
 }
 
-void Database::updateOlmSession(const QString& senderKey,
+void Database::updateOlmSession(const QByteArray& senderKey,
                                 const QOlmSession& session)
 {
     auto query = prepareQuery(
